@@ -174,10 +174,11 @@ class interface(abstract_instrument_interface.abstract_interface):
         #position and update it in the GUI. When it becomes False, call self.end_movement
         self.check_property_until(self.instrument,pyThorlabsAPT.is_in_motion,[True,False],[[self.read_position],[self.end_movement]])
         
-    def end_movement(self):
+    def end_movement(self,reset_gui = True):
         self.read_position()
         self.logger.info(f"Movement ended. New position = {self.output['Position']}")
-        self.gui.set_non_moving_state()
+        if reset_gui:
+            self.gui.set_non_moving_state()
 
     def read_stage_info(self):
         #From the thorlabs_APT code:
@@ -241,27 +242,11 @@ class interface(abstract_instrument_interface.abstract_interface):
         self.settings['ramp_repeat'] = repeat_ramp
         self.settings['ramp_reverse'] = add_reverse
 
-        #generate a list of actions that define the ramp
-        action =[]
-        if initial_trigger:
-            action.append({'action':'send_trigger'})
-            action.append({'action':'wait', 'time':wait2})
-        for j in range(repeat_ramp): #when repeat_ramp > 1, the whole ramp is repeated multiple times
-            for i in range(numb_steps):
-                action.append({'action':'move', 'stepsize':stepsize})
-                action.append({'action':'wait', 'time':wait1})
-                action.append({'action':'send_trigger'})
-                action.append({'action':'wait', 'time':wait2})
-            if add_reverse:
-                for i in range(numb_steps):
-                    action.append({'action':'move', 'stepsize':-stepsize})
-                    action.append({'action':'wait', 'time':wait1})
-                    action.append({'action':'send_trigger'})
-                    action.append({'action':'wait', 'time':wait2})
+        actions = self.generate_list_actions(initial_trigger, stepsize, wait1, wait2, numb_steps, add_reverse , repeat_ramp)
         self.logger.info(f"Starting ramp...")
         self.gui.set_doingramp_state()
         self.doing_ramp = True
-        self.run_sequence(action)
+        self.run_sequence(actions)
     
     def stop_ramp(self):
         self.doing_ramp = False
@@ -288,7 +273,7 @@ class interface(abstract_instrument_interface.abstract_interface):
             #position and update it in the GUI. When it becomes False, call self.end_movement
             self.check_property_until(self.instrument,pyThorlabsAPT.is_in_motion,[True,False],[
                                                                                                 [self.read_position],
-                                                                                                [self.end_movement,lambda: self._run_sequence(index+1)]
+                                                                                                [lambda:self.end_movement(reset_gui=False),lambda: self._run_sequence(index+1)]
                                                                                                 ])
         if current_action['action'] == 'wait':
             self.logger.info(f"Waiting for {float(current_action['time'])} s...")
@@ -310,17 +295,16 @@ class gui(abstract_instrument_interface.abstract_gui):
 
         ### SET INITIAL STATE OF WIDGETS
         self.edit_StepSize.setText(str(self.interface.settings['step_size']))
-        self.checkbox_Initial_trigger.setChecked(bool(self.interface.settings['ramp_send_initial_trigger']))
-        self.edit_StepSize_ramp.setText(str(self.interface.settings['ramp_step_size']))
-        self.edit_Wait1_ramp.setText(str(self.interface.settings['ramp_wait_1']))
-        self.edit_Wait2_ramp.setText(str(self.interface.settings['ramp_wait_2']))
-        self.spinbox_Ramp_steps.setValue(int(self.interface.settings[ 'ramp_numb_steps']))
-        self.checkbox_Reverse.setChecked(bool(self.interface.settings['ramp_reverse']))
-        self.spinbox_Ramp_repeat.setValue(int(self.interface.settings[ 'ramp_repeat']))
+        self.ramp_groupbox.checkbox_Initial_trigger.setChecked(bool(self.interface.settings['ramp_send_initial_trigger']))
+        self.ramp_groupbox.edit_StepSize.setText(str(self.interface.settings['ramp_step_size']))
+        self.ramp_groupbox.edit_Wait1.setText(str(self.interface.settings['ramp_wait_1']))
+        self.ramp_groupbox.edit_Wait2.setText(str(self.interface.settings['ramp_wait_2']))
+        self.ramp_groupbox .spinbox_steps.setValue(int(self.interface.settings[ 'ramp_numb_steps']))
+        self.ramp_groupbox.checkbox_Reverse.setChecked(bool(self.interface.settings['ramp_reverse']))
+        self.ramp_groupbox.spinbox_repeat.setValue(int(self.interface.settings[ 'ramp_repeat']))
         self.click_button_refresh_list_devices()    #By calling this method, as soon as the gui is created we also look for devices
         self.set_disconnected_state()               #When GUI is created, all widgets are set to the "Disconnected" state
         ###
-
         self.connect_widgets_events_to_functions()
 
         ### Call the initialize method of the parent class
@@ -379,49 +363,12 @@ class gui(abstract_instrument_interface.abstract_gui):
             stageparams_hbox.addWidget(w)
         stageparams_groupbox.setLayout(stageparams_hbox) 
         
-        ramp_groupbox = Qt.QGroupBox()
-        ramp_groupbox.setTitle(f"Ramp")
-        ramp_vbox = Qt.QVBoxLayout()
-        ramp_hbox1 = Qt.QHBoxLayout()
-        ramp_hbox2 = Qt.QHBoxLayout()
-        self.checkbox_Initial_trigger = Qt.QCheckBox("Send initial trigger (+wait)")
-        tooltip = 'When this interface is used within a larger software, it can be set to send a trigger (to another function) everytime a step of the ramp is done (see documentation).\nBy ticking this on, a trigger is sent also at the beginning of the ramp.'
-        self.checkbox_Initial_trigger.setToolTip(tooltip)
-        self.label_Move_ramp = Qt.QLabel("Move by")
-        self.edit_StepSize_ramp = Qt.QLineEdit()
-        self.label_Wait1_ramp = Qt.QLabel(",wait for")
-        self.edit_Wait1_ramp = Qt.QLineEdit()
-        self.edit_Wait1_ramp.setMaximumWidth(35)
-        self.label_Wait2_ramp = Qt.QLabel("s, send trigger, wait for")
-        self.edit_Wait2_ramp = Qt.QLineEdit()
-        self.edit_Wait2_ramp.setMaximumWidth(35)
-        self.label_Ramp_steps = Qt.QLabel("s, repeat")
-        self.spinbox_Ramp_steps = Qt.QSpinBox()
-        self.spinbox_Ramp_steps.setRange(1, 100000)
-        self.label_Ramp_steps2 = Qt.QLabel("times.")
-        for w in [self.checkbox_Initial_trigger,
-                  self.label_Move_ramp,self.edit_StepSize_ramp,self.label_Wait1_ramp,
-                  self.edit_Wait1_ramp,self.label_Wait2_ramp,
-                  self.edit_Wait2_ramp,self.label_Ramp_steps,self.spinbox_Ramp_steps,self.label_Ramp_steps2]:
-            ramp_hbox1.addWidget(w)
-
-        self.checkbox_Reverse = Qt.QCheckBox("and reverse.")
-        self.label_Ramp_repeat = Qt.QLabel(" Repeat ramp")
-        self.spinbox_Ramp_repeat = Qt.QSpinBox()
-        self.spinbox_Ramp_repeat.setRange(1, 100000)
-        self.label_Ramp_repeat2 = Qt.QLabel(" times.")
-        self.button_StartRamp = Qt.QPushButton("Start Ramp")
-        for w in [self.checkbox_Reverse,self.label_Ramp_repeat,self.spinbox_Ramp_repeat,self.label_Ramp_repeat2,self.button_StartRamp]:
-            ramp_hbox2.addWidget(w)
-        ramp_vbox.addLayout(ramp_hbox1)  
-        ramp_vbox.addLayout(ramp_hbox2)  
-        ramp_groupbox.setLayout(ramp_vbox ) 
-        #hbox2.addStretch(1)
+        self.ramp_groupbox = abstract_instrument_interface.ramp_panel()
                 
         self.container = Qt.QVBoxLayout()
         for box in [hbox1,hbox2]:
             self.container.addLayout(box)  
-        self.container.addWidget(ramp_groupbox)
+        self.container.addWidget(self.ramp_groupbox)
         self.container.addWidget(stageparams_groupbox)
 
         self.container.addStretch(1)
@@ -429,23 +376,21 @@ class gui(abstract_instrument_interface.abstract_gui):
         self.widgets_disabled_when_doing_ramp = [self.button_ConnectDevice,self.combo_Devices,
                                                  self.label_Position,self.edit_Position,self.button_Home,
                                                  self.label_min_pos,self.edit_min_pos,self.label_max_pos,self.edit_max_pos,self.label_units,self.combo_units,self.label_pitch,self.edit_pitch,self.button_set_stageparams,
-                                               self.label_Move,self.button_MoveNegative,self.button_MovePositive,self.label_By,self.edit_StepSize,
-                                               self.checkbox_Initial_trigger,self.label_Move_ramp,self.edit_StepSize_ramp,self.label_Wait1_ramp,self.edit_Wait1_ramp,self.label_Wait2_ramp,
-                                               self.edit_Wait2_ramp,self.label_Ramp_steps,self.spinbox_Ramp_steps,self.label_Ramp_steps2,
-                                               self.checkbox_Reverse,self.label_Ramp_repeat,self.spinbox_Ramp_repeat,self.label_Ramp_repeat2]
-
+                                               self.label_Move,self.button_MoveNegative,self.button_MovePositive,self.label_By,self.edit_StepSize
+                                               ]
+        #These widgets are enabled ONLY when interface is connected to a device
         self.widgets_enabled_when_connected = [self.combo_Devices , self.button_RefreshDeviceList,
                                                self.label_Position, self.edit_Position,self.button_Home,
                                                self.label_min_pos,self.edit_min_pos,self.label_max_pos,self.edit_max_pos,self.label_units,self.combo_units,self.label_pitch,self.edit_pitch,self.button_set_stageparams,
                                                self.label_Move,self.button_MoveNegative,self.button_MovePositive,self.label_By,self.edit_StepSize,
-                                               self.checkbox_Initial_trigger,self.label_Move_ramp,self.edit_StepSize_ramp,self.label_Wait1_ramp,self.edit_Wait1_ramp,self.label_Wait2_ramp,
-                                               self.edit_Wait2_ramp,self.label_Ramp_steps,self.spinbox_Ramp_steps,self.label_Ramp_steps2,
-                                               self.checkbox_Reverse,self.label_Ramp_repeat,self.spinbox_Ramp_repeat,self.label_Ramp_repeat2,self.button_StartRamp]
+                                               self.ramp_groupbox
+                                               ]
 
+        #These widgets are enabled ONLY when interface is NOT connected to a device   
         self.widgets_enabled_when_disconnected = [self.combo_Devices , 
                                                   self.button_RefreshDeviceList]
 
-        self.widgets_disabled_when_moving = [self.button_StartRamp,self.button_set_stageparams,self.button_MoveNegative ,self.button_MovePositive,self.button_Home]
+        self.widgets_disabled_when_moving = [self.ramp_groupbox,self.button_set_stageparams,self.button_MoveNegative ,self.button_MovePositive,self.button_Home]
 
     def connect_widgets_events_to_functions(self):
         self.button_RefreshDeviceList.clicked.connect(self.click_button_refresh_list_devices)
@@ -456,7 +401,7 @@ class gui(abstract_instrument_interface.abstract_gui):
         self.button_Home.clicked.connect(self.click_button_Home)
         self.edit_StepSize.returnPressed.connect(self.press_enter_edit_StepSize)
         self.button_set_stageparams .clicked.connect(self.click_button_set_stageparams)
-        self.button_StartRamp.clicked.connect(self.click_button_start_ramp)
+        self.ramp_groupbox.button_StartRamp.clicked.connect(self.click_button_start_ramp)
 
     def set_disconnecting_state(self):
         self.disable_widget(self.widgets_enabled_when_connected)
@@ -477,14 +422,6 @@ class gui(abstract_instrument_interface.abstract_gui):
         self.enable_widget(self.widgets_enabled_when_connected)
         self.disable_widget(self.widgets_enabled_when_disconnected)
         self.button_ConnectDevice.setText("Disconnect")
-        
-    def set_doingramp_state(self):
-        self.disable_widget(self.widgets_disabled_when_doing_ramp )
-        self.button_StartRamp.setText("Stop Ramp")
-        
-    def set_notdoingramp_state(self):
-        self.enable_widget(self.widgets_disabled_when_doing_ramp )
-        self.button_StartRamp.setText("Start Ramp")
 
     def set_moving_state(self):
         self.disable_widget(self.widgets_disabled_when_moving)
@@ -492,6 +429,15 @@ class gui(abstract_instrument_interface.abstract_gui):
     def set_non_moving_state(self):
         self.enable_widget(self.widgets_disabled_when_moving)
 
+    def set_doingramp_state(self):
+        self.disable_widget(self.widgets_disabled_when_doing_ramp )
+        self.ramp_groupbox.set_doingramp_state()
+        
+    def set_notdoingramp_state(self):
+        self.enable_widget(self.widgets_disabled_when_doing_ramp )
+        self.set_connected_state()
+        self.ramp_groupbox.set_notdoingramp_state()
+
 ############################################################
 ### GUI Events Functions
 ############################################################
@@ -514,18 +460,17 @@ class gui(abstract_instrument_interface.abstract_gui):
     
     def click_button_start_ramp(self):
         if self.interface.doing_ramp == False:
-            self.interface.start_ramp(initial_trigger = (self.checkbox_Initial_trigger.isChecked() == True),
-                                      stepsize =   float(self.edit_StepSize_ramp.text()),
-                                      wait1 =       float(self.edit_Wait1_ramp.text()),
-                                      wait2 =       float(self.edit_Wait2_ramp.text()),
-                                      numb_steps =     int(self.spinbox_Ramp_steps.value()),
-                                      add_reverse = self.checkbox_Reverse.isChecked(),
-                                      repeat_ramp = int(self.spinbox_Ramp_repeat.value())
+            self.interface.start_ramp(initial_trigger = (self.ramp_groupbox.checkbox_Initial_trigger.isChecked() == True),
+                                      stepsize =   float(self.ramp_groupbox.edit_StepSize.text()),
+                                      wait1 =       float(self.ramp_groupbox.edit_Wait1.text()),
+                                      wait2 =       float(self.ramp_groupbox.edit_Wait2.text()),
+                                      numb_steps =     int(self.ramp_groupbox.spinbox_steps.value()),
+                                      add_reverse = self.ramp_groupbox.checkbox_Reverse.isChecked(),
+                                      repeat_ramp = int(self.ramp_groupbox.spinbox_repeat.value())
                                       )
         else:
             self.interface.stop_ramp()
 
-        
     def click_button_Move(self,direction):
         self.press_enter_edit_StepSize()
         self.interface.move_single_step(direction)
